@@ -1,16 +1,18 @@
 # -*- coding: utf-8 -*-
-
 from qa4sm_reader import globals
-from qa4sm_reader.handlers import QA4SMDatasets, QA4SMMetricVariable, QA4SMMetric
-from qa4sm_reader.plot_utils import _format_floats
+import qa4sm_reader.handlers as hdl
+from qa4sm_reader.plotting_methods import _format_floats, combine_soils, combine_depths
 
 from parse import *
 from pathlib import Path
-import numpy as np
-import xarray as xr
 from collections import OrderedDict
 import itertools
+import warnings
+
+import numpy as np
+import xarray as xr
 import pandas as pd
+from typing import Union
 
 
 def extract_periods(filepath) -> np.array:
@@ -69,7 +71,7 @@ class QA4SMImg(object):
         self.ignore_empty = ignore_empty
         self.ds = self._open_ds(extent=extent, period=period, engine=engine)
         self.extent = self._get_extent(extent=extent)  # get extent from .nc file if not specified
-        self.datasets = QA4SMDatasets(self.ds.attrs)
+        self.datasets = hdl.QA4SMDatasets(self.ds.attrs)
 
         if load_data:
             self.varnames = list(self.ds.variables.keys())
@@ -80,7 +82,7 @@ class QA4SMImg(object):
             # this try here is to obey tests, withouth a necessity of changing and commiting test files again
             try:
                 self.ref_dataset_grid_stepsize = self.ds.val_dc_dataset0_grid_stepsize
-            except:
+            except AttributeError:
                 self.ref_dataset_grid_stepsize = 'nan'
 
     def _open_ds(self, extent=None, period=None, engine='h5netcdf'):
@@ -116,12 +118,9 @@ class QA4SMImg(object):
     @property
     def has_CIs(self):
         """True if the validation result contains confidence intervals"""
-        cis = False
-        # check if there is any CI Var
-        for Var in self._iter_vars():
-            if Var.is_CI:
-                cis = True
-        return cis
+        itdoes = hdl.ConfidenceInterval in [type(var) for var in self.vars]
+
+        return itdoes
 
     @property
     def name(self) -> str:
@@ -133,6 +132,38 @@ class QA4SMImg(object):
 
         return name
 
+    @property
+    def metadata(self) -> dict:
+        """If the image has metadata (ISMN reference), return a dict of shape {varname: Metadata}. Else, False."""
+        metadata = {}
+        # check if there is any CI Var
+        for Var in self._iter_vars(type="metadata"):
+            metadata[Var.varname] = Var
+
+        # metadata that are generated upon initialization (soil type and instrument depth):
+        # Do not provide else statement to avoid dealing with None further on
+        if all(type in metadata.keys() for type in globals.soil_types):
+            soil_dict = {type: metadata[type] for type in globals.soil_types}
+            soil_combined = combine_soils(soil_dict)
+            metadata["soil_type"] = hdl.QA4SMVariable("soil_type", self.ds.attrs, values=soil_combined).initialize()
+
+        else:
+            warnings.warn(
+                "Not all: " + ", ".join(globals.soil_types) + " are present in the netCDF variables"
+            )
+
+        if all(type in metadata.keys() for type in globals.instrument_depths):
+            depth_dict = {type: metadata[type] for type in globals.instrument_depths}
+            depth_combined = combine_depths(depth_dict)
+            metadata["instrument_depth"] = hdl.QA4SMVariable("instrument_depth", self.ds.attrs, values=depth_combined).initialize()
+
+        else:
+            warnings.warn(
+                "Not all: " + ", ".join(globals.instrument_depths) + " are present in the netCDF variables"
+            )
+
+        return metadata
+
     def _get_extent(self, extent) -> tuple:
         """Get extent of the results from the netCDF file"""
         if not extent:
@@ -158,7 +189,7 @@ class QA4SMImg(object):
         Returns
         -------
         vars : list
-            list of QA4SMMetricVariable objects for the validation variables
+            list of QA4SMVariable objects for the validation variables
         """
         vars = []
         for varname in self.varnames:
@@ -171,19 +202,12 @@ class QA4SMImg(object):
                 except KeyError:
                     values = None
 
-            try:
-                Var = QA4SMMetricVariable(varname, self.ds.attrs, values=values)
-                # if self.ignore_empty and Var.isempty: todo: possible issues from non-metric variables?
-                #     continue
-            except IOError:
-                Var = None
-                continue
+            Var = hdl.QA4SMVariable(varname, self.ds.attrs, values=values).initialize()
 
-            if Var is not None:
-                if only_metrics and Var.ismetric:
-                    vars.append(Var)
-                elif not only_metrics:
-                    vars.append(Var)
+            if only_metrics and isinstance(Var, hdl.MetricVariable):
+                vars.append(Var)
+            elif not only_metrics:
+                vars.append(Var)
 
         return vars
 
@@ -201,29 +225,41 @@ class QA4SMImg(object):
         for group in all_groups:
             for metric in group:
                 metric_vars = []
-                for Var in self._iter_vars(**{'metric': metric}):
+                for Var in self._iter_vars(filter_parms={'metric': metric}):
                     metric_vars.append(Var)
 
                 if metric_vars != []:
-                    Metric = QA4SMMetric(metric, metric_vars)
+                    Metric = hdl.QA4SMMetric(metric, metric_vars)
                     Metrics[metric] = Metric
 
         return Metrics
 
-    def _iter_vars(self, only_metrics=False, **filter_parms) -> iter:
+    def _iter_vars(self, type:str=None, name:str=None, filter_parms:dict=None) -> iter:
         """
-        Iter through QA4SMMetricVariable objects that are in the file
+        Iter through QA4SMVariable objects that are in the file
 
         Parameters
         ----------
-        only_metrics: bool, optional. Default is Fales.
-            If True, only Vars that belong to a group are taken
-        **filter_parms : kwargs, dict
-            dictionary with QA4SMMetricVariable attributes as keys and filter value as values (e.g. {g: 0})
+        type : str, default is None
+            One of 'metric', 'ci', 'metadata' can be specified to only iterate through the specific group
+        name : str, default is None
+            yield a specific variable by its name
+        filter_parms : dict
+            dictionary with QA4SMVariable attributes as keys and filter value as values (e.g. {g: 0})
         """
+        type_lut = {
+            "metric": hdl.MetricVariable,
+            "ci": hdl.ConfidenceInterval,
+            "metadata": hdl.Metadata,
+        }
         for Var in self.vars:
-            if only_metrics:
-                if Var.g is None:
+            if name:
+                if name in [Var.varname, Var.pretty_name]:
+                    yield Var
+                    break
+                else:
+                    continue
+            if type and not isinstance(Var, type_lut[type]):
                     continue
             if filter_parms:
                 for key, val in filter_parms.items():
@@ -232,10 +268,10 @@ class QA4SMImg(object):
                     else:
                         check = False  # does not match requirements
                         break
-                if check == True:
-                    yield Var
-            else:
-                yield Var
+                if check != True:
+                    continue
+
+            yield Var
 
     def _iter_metrics(self, **filter_parms) -> iter:
         """
@@ -251,17 +287,17 @@ class QA4SMImg(object):
                 if val is None or getattr(Metric, key) == val:
                     yield Metric
 
-    def group_vars(self, **filter_parms):
+    def group_vars(self, filter_parms:dict):
         """
-        Return a list of QA4SMMetricVariable that match filters
+        Return a list of QA4SMVariable that match filters
 
         Parameters
         ----------
         **filter_parms : kwargs, dict
-            dictionary with QA4SMMetricVariable attributes as keys and filter value as values (e.g. {g: 0})
+            dictionary with QA4SMVariable attributes as keys and filter value as values (e.g. {g: 0})
         """
         vars = []
-        for Var in self._iter_vars(**filter_parms):
+        for Var in self._iter_vars(filter_parms=filter_parms):
             vars.append(Var)
 
         return vars
@@ -300,7 +336,7 @@ class QA4SMImg(object):
         Parameters
         ----------
         varnames : list or None
-            list of QA4SMMetricVariables to be placed in the DataFrame
+            list of QA4SMVariables to be placed in the DataFrame
 
         Return
         ------
@@ -323,9 +359,10 @@ class QA4SMImg(object):
             lat, lon, gpi = globals.index_names
             df[lat] = df.index.get_level_values(lat)
             df[lon] = df.index.get_level_values(lon)
+
             if gpi in df.index:
                 df[gpi] = df.index.get_level_values(gpi)
-        # import pdb; pdb.set_trace()
+
         df.reset_index(drop=True, inplace=True)
         df = df.set_index(self.index_names)
 
@@ -349,15 +386,34 @@ class QA4SMImg(object):
         if isinstance(metrics, list):
             Vars = []
             for metric in metrics:
-                Vars.extend(self.group_vars(**{'metric':metric}))
+                Vars.extend(self.group_vars(filter_parms={'metric':metric}))
         else:
-            Vars = self.group_vars(**{'metric':metrics})
+            Vars = self.group_vars(filter_parms={'metric':metrics})
 
         varnames = [Var.varname for Var in Vars]
         metrics_df = self._ds2df(varnames=varnames)
 
         return metrics_df
 
+    def get_cis(self, Var:hdl.MetricVariable) -> Union[list, None]:
+        """Return the CIs of a variable as a list of dfs ('upper' and 'lower'), if they exist in the netcdf"""
+        cis = []
+        if not self.has_CIs:
+            return cis
+        for ci in self._iter_vars(
+                type="ci",
+                filter_parms={
+                    "metric":Var.metric,
+                    "metric_ds":Var.metric_ds,
+                    "other_ds":Var.other_ds,
+                }
+        ):
+            values = ci.values
+            values.columns = [ci.bound]
+            cis.append(values)
+
+        return cis
+
     def _metric_stats(self, metric, id=None)  -> list:
         """
         Provide a list with the metric summary statistics for each variable or for all variables
@@ -376,12 +432,11 @@ class QA4SMImg(object):
             List of (variable) lists with summary statistics
         """
         metric_stats = []
+        filters = {'metric':metric}
         if id:
-            filters = {'metric':metric, 'is_CI':False, 'id':id}
-        else:
-            filters = {'metric':metric, 'is_CI':False,}
+            filters.update(id=id)
         # get stats by metric
-        for Var in self._iter_vars(only_metrics=True, **filters):
+        for Var in self._iter_vars(type="metric", filter_parms=filters):
             # get interquartile range 
             values = Var.values[Var.varname]
             # take out variables with all NaN or NaNf
